<?php if (!defined('PmWiki')) exit();
/*
 * Copyright 2006 Kathryn Andersen
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the Gnu Public Licence or the Artistic Licence.
 */ 

/** \file storyarchive.php
 * \brief things to support a story archive
 *
 * See Also: http://www.pmwiki.org/wiki/Cookbook/StoryArchive
 *
 * This script provides a number of page variables, as well as supporting
 * pagelist templates and CSS markup, to aid in creating a story archive
 * in PmWiki.
 *
 * This requires 2.2-beta15 or later.
 *
 * To activate this script, copy it into the cookbook/ directory, then add
 * the following line to your local/config.php:
 *
 *      include_once("$FarmD/cookbook/storyarchive.php");
 * 
*/

$RecipeInfo['StoryArchive']['Version'] = '20061201';

SDV($StoryArchivePubDir, '$FarmPubDir/storyarchive');
SDV($StoryArchivePubDirUrl, '$FarmPubDirUrl/storyarchive');
SDV($StoryArchiveLibDir, '$FarmD/cookbook/storyarchive/storylib.d');
SDV($StoryArchiveGroupSkipPattern, '(Title)');

#--------------------------------------------
# add CSS
$HTMLHeaderFmt['storyarchive'] = 
   "<link rel='stylesheet' href='\$StoryArchivePubDirUrl/storyarchive.css' 
     type='text/css' />
     ";

#------------------------------------------
## automatic loading of story-archive pages
global $WikiLibDirs;
$where = count($WikiLibDirs);
if ($where>1) $where--;
array_splice($WikiLibDirs, $where, 0, 
	     array(new PageStore("$StoryArchiveLibDir/\$FullName")));

#------------------------------------------
## automatic loading of story-archive pagelist templates
global $FPLTemplatePageFmt;
$where = count($FPLTemplatePageFmt);
if ($where>1) $where--;
if ($where > 0) {
    array_splice($FPLTemplatePageFmt, $where, 0, 
		 array('{$SiteGroup}.StoryArchiveTemplates'));
}
else {
    SDV($FPLTemplatePageFmt, array('{$FullName}',
				   '{$SiteGroup}.LocalTemplates',
				   '{$SiteGroup}.PageListTemplates',
				   '{$SiteGroup}.StoryArchiveTemplates'));
}

# ----------------- Templates -----------------------------------
# multiple templates with (:edittemplate:) directive
# change the link format for creating new pages
$LinkPageCreateFmt = "<a class='createlinktext'
    href='\$PageUrl?action=edit\$LinkTemplate'>\$LinkText</a>
    <a class='createlink' href='\$PageUrl?action=edit\$LinkTemplate'>?</a>";

# default is no template
$FmtV['$LinkTemplate'] = '';

# the (:edittemplate Group.PageName:) markup causes any create page 
# links that follow to use Group.PageName as the template
Markup('edittemplate', 'directives',
    '/\\(:edittemplate(\\s+(\\S+))?\\s*:\\)/e',
    "PZZ(\$GLOBALS['FmtV']['\$LinkTemplate'] = '$2' ? '&amp;template=$2' : '')");

# --------------------- pagelist/search -------------------------
# 'content' includes self
$SearchPatterns['content']['recent'] = '!\.(All)?Recent(Changes|Uploads)$!';
$SearchPatterns['content']['group'] = '!\.Group(Print)?(Header|Footer|Attributes)$!';

# list all stories
$SearchPatterns['stories']['storygroup'] = '/^Stories-/';
$SearchPatterns['stories']['nonhome'] = '!\.Stories-!';
$SearchPatterns['stories']['recent'] = '!\.(All)?Recent(Changes|Uploads)$!';
$SearchPatterns['stories']['group'] = '!\.Group(Print)?(Header|Footer|Attributes)$!';
$SearchPatterns['stories']['side'] = '!\.(Side|Menu)Bar$!';
$SearchPatterns['stories']['breadcrumb'] = '!\.BreadCrumb!';
$SearchPatterns['stories']['template'] = '!Template$!';
$SearchPatterns['stories']['comment'] = '!-Comment(s)?$!';

/*======================================================================
 * Page Variables
 */
# {$PageCreationDate} in format yyyy-mm-dd
$FmtPV['$PageCreationDate'] = 'strftime("%Y-%m-%d", $page["ctime"])';
$FmtPV['$PageCreationTime'] = 'strftime("%Y-%m-%d %H:%M", $page["ctime"])';

$FmtPV['$TitleA'] = '@$page["title"] ? substr($page["title"], 0, 1) : substr($name, 0, 1)';
$FmtPV['$NameA'] = 'substr($name, 0, 1)';
$FmtPV['$GroupA'] = 'substr($group, 0, 1)';

$FmtPV['$Name1'] = 'StoryArchiveSplitName($name, 0)';
$FmtPV['$Name2'] = 'StoryArchiveSplitName($name, 1)';
$FmtPV['$Name3'] = 'StoryArchiveSplitName($name, 2)';

$FmtPV['$Group1'] = 'StoryArchiveSplitName($group, 0)';
$FmtPV['$Group2'] = 'StoryArchiveSplitName($group, 1)';

$FmtPV['$Date'] = 'StoryArchiveGetDate($pn)';
$FmtPV['$Year'] = 'StoryArchiveSplitName(StoryArchiveGetDate($pn), 0)';
$FmtPV['$Month'] = 'sprintf("%02d", StoryArchiveSplitName(StoryArchiveGetDate($pn), 1))';
$FmtPV['$Day'] = 'sprintf("%02d", StoryArchiveSplitName(StoryArchiveGetDate($pn), 2))';
$FmtPV['$MonthName'] = 'StoryArchiveGetMonthName($pn)';

$FmtPV['$SeriesOrdering'] = 
	'sprintf("%s-%03d-%0.3f-%s", PageTextVar($pn, "Series"),  PageTextVar($pn, "SeriesOrder"), PageTextVar($pn, "SeriesOrder"), $name)';

/*======================================================================
 * Markup
 */
## (:story:)
Markup('story', '<table',
       '/\\(:story(end)?:\\)/ei',
       "StoryArchiveInfoMarkup('$1')");

Markup('^StoryPropertyLink:', '<inline',
       '/^([A-Z][-\\w]+):\\s*(.*)?$/ei',
       "StoryArchiveInfoPropertyLinkMarkup(\$pagename,'$1','$2','$0')");

/*======================================================================
 * Functions
 */

# get the date, either from the Date: page text var
# or from the ctime of the page
function StoryArchiveGetDate($pagename) {
    $date = PageTextVar($pagename, "Date");
    if (!$date) {
	$page = RetrieveAuthPage($pagename, 'read', false, READPAGE_CURRENT);
	$date  = strftime("%Y-%m-%d", $page["ctime"]);
    }
    return $date;
}

// split a name on '-' and return the requested bit
function StoryArchiveSplitName($name, $ind) {
    $bits = split('-', $name);
    return $bits[$ind];
}

function StoryArchiveGetMonthName ($pagename) {
    $month_num = StoryArchiveSplitName(StoryArchiveGetDate($pagename), 1);
    if ($month_num == 1) {
	return 'January';
    } else if ($month_num == 2) {
	return 'February';
    } else if ($month_num == 3) {
	return 'March';
    } else if ($month_num == 4) {
	return 'April';
    } else if ($month_num == 5) {
	return 'May';
    } else if ($month_num == 6) {
	return 'June';
    } else if ($month_num == 7) {
	return 'July';
    } else if ($month_num == 8) {
	return 'August';
    } else if ($month_num == 9) {
	return 'September';
    } else if ($month_num == 10) {
	return 'October';
    } else if ($month_num == 11) {
	return 'November';
    } else if ($month_num == 12) {
	return 'December';
    }
    return '';
}

function StoryArchiveInfoMarkup($x) {
    $out = '';
    if ($x == 'end') {
	$GLOBALS['StoryArchiveInfoMarkupOn'] = false;
	$GLOBALS['HTMLPNewline'] = '';
	$out = '(:divend:)';
    }
    else {
	$GLOBALS['StoryArchiveInfoMarkupOn'] = true;
	$GLOBALS['HTMLPNewline'] = '<br  />';
	$out = '(:div class="story":)';
    }
    return $out;
}

function StoryArchiveInfoPropertyLinkMarkup($pagename, $property, $rest_of_line,$whole_line) {
    global $StoryArchiveInfoMarkupOn;
    global $StoryArchiveGroupSkipPattern;
    if ($StoryArchiveInfoMarkupOn) {
	$prop_part = "'''" . $property . ":''' ";
	$group_page = $property . '.' . $property;
	if (PageExists($group_page)
	    && preg_match("/$StoryArchiveGroupSkipPattern/", $property) == false
	   ) {
	    // if it has a link, don't change it
	    if (preg_match('/\[\[/', $rest_of_line)) {
		return $prop_part . $rest_of_line;
	    }
	    else {
		return $prop_part . "[[$property/$rest_of_line]]";
	    }
	}
	else {
	    return $prop_part . $rest_of_line;
	}
    } else {
	return $whole_line;
    }
}

